@@ -545,6 +545,8 @@ exchange_folder_get_message_sync (CamelFolder *folder,
 	if (!ba)
 		return NULL;
 
+	g_byte_array_append (ba, (guint8 *) "", 1);
+
 	while ((ba->len > 10 && g_str_has_prefix ((const gchar *)ba->data, "MAIL FROM:")) ||
 	       (ba->len >  8 && g_str_has_prefix ((const gchar *)ba->data, "RCPT TO:")) ||
 	       (ba->len >  2 && (ba->data[0] == '\n' || ba->data[1] == '\n'))) {
@@ -560,6 +562,9 @@ exchange_folder_get_message_sync (CamelFolder *folder,
 			break;
 	}
 
+	if (ba->len > 0 && ba->data[ba->len - 1] == 0)
+		g_byte_array_remove_index (ba, ba->len - 1);
+
 	stream = camel_stream_mem_new_with_byte_array (ba);
 
 	crlffilter = camel_mime_filter_crlf_new (CAMEL_MIME_FILTER_CRLF_DECODE, CAMEL_MIME_FILTER_CRLF_MODE_CRLF_ONLY);
